@@ -229,6 +229,13 @@ class BasePlot(qwt.QwtPlot):
     #:     plot: the plot
     SIG_ITEMS_CHANGED = QC.Signal(object)
 
+    #: Signal emitted by plot when item parameters have changed (through the item's
+    #: parameters dialog, or when setting colormap using the dedicated tool)
+    #:
+    #: Args:
+    #:     item: the item
+    SIG_ITEM_PARAMETERS_CHANGED = QC.Signal(object)
+
     #: Signal emitted by plot when selected item has changed
     #:
     #: Args:
@@ -434,7 +441,7 @@ class BasePlot(qwt.QwtPlot):
                 # This happens when object has already been deleted
                 pass
 
-    def on_active_curve(self, x: float, y: float) -> None:
+    def on_active_curve(self, x: float, y: float) -> tuple[float, float]:
         """
         Callback called when the active curve is moved
 
@@ -442,7 +449,7 @@ class BasePlot(qwt.QwtPlot):
             x (float): the x position
             y (float): the y position
         """
-        curve: CurveItem = self.get_last_active_item(itf.ITrackableItemType)
+        curve: CurveItem = self.get_last_active_item(itf.ICurveItemType)
         if curve:
             x, y = curve.get_closest_coordinates(x, y)
         return x, y
@@ -542,6 +549,7 @@ class BasePlot(qwt.QwtPlot):
         self,
         dx: tuple[float, float, float, float],
         dy: tuple[float, float, float, float],
+        replot: bool = True,
     ) -> None:
         """
         Translate the active axes according to dx, dy axis 'state' tuples
@@ -549,6 +557,8 @@ class BasePlot(qwt.QwtPlot):
         Args:
             dx (tuple[float, float, float, float]): the x axis 'state' tuple
             dy (tuple[float, float, float, float]): the y axis 'state' tuple
+            replot: if True, do a full replot else just update the axes to avoid a
+             redraw (default: True)
         """
         # dx and dy are the output of the "DragHandler.get_move_state" method
         # (see module ``plotpy.events``)
@@ -567,7 +577,10 @@ class BasePlot(qwt.QwtPlot):
             self.set_axis_limits(axis_id, vmin, vmax)
 
         self.setAutoReplot(auto)
-        self.replot()
+        if replot:
+            self.replot()
+        else:
+            self.updateAxes()
         # the signal MUST be emitted after replot, otherwise
         # we receiver won't see the new bounds (don't know why?)
         self.SIG_PLOT_AXIS_CHANGED.emit(self)
@@ -577,12 +590,20 @@ class BasePlot(qwt.QwtPlot):
         dx: tuple[float, float, float, float],
         dy: tuple[float, float, float, float],
         lock_aspect_ratio: bool | None = None,
+        replot: bool = True,
     ) -> None:
         """
         Change the scale of the active axes (zoom/dezoom) according to dx, dy
         axis 'state' tuples
 
         We try to keep initial pos fixed on the canvas as the scale changes
+
+        Args:
+            dx (tuple[float, float, float, float]): the x axis 'state' tuple
+            dy (tuple[float, float, float, float]): the y axis 'state' tuple
+            lock_aspect_ratio: if True, the aspect ratio is locked
+            replot: if True, do a full replot else just update the axes to avoid a
+             redraw (default: True)
         """
         # dx and dy are the output of the "DragHandler.get_move_state" method
         # (see module ``plotpy.events``):
@@ -627,7 +648,10 @@ class BasePlot(qwt.QwtPlot):
             self.set_axis_limits(axis_id, vmin, vmax)
 
         self.setAutoReplot(auto)
-        self.replot()
+        if replot:
+            self.replot()
+        else:
+            self.updateAxes()
         # the signal MUST be emitted after replot, otherwise
         # we receiver won't see the new bounds (don't know why?)
         self.SIG_PLOT_AXIS_CHANGED.emit(self)
@@ -1443,9 +1467,11 @@ class BasePlot(qwt.QwtPlot):
 
     def serialize(
         self,
-        writer: guidata.dataset.io.HDF5Writer
-        | guidata.dataset.io.INIWriter
-        | guidata.dataset.io.JSONWriter,
+        writer: (
+            guidata.dataset.io.HDF5Writer
+            | guidata.dataset.io.INIWriter
+            | guidata.dataset.io.JSONWriter
+        ),
     ) -> None:
         """Serialize object to HDF5 writer
 
@@ -1459,9 +1485,11 @@ class BasePlot(qwt.QwtPlot):
 
     def deserialize(
         self,
-        reader: guidata.dataset.io.HDF5Reader
-        | guidata.dataset.io.INIReader
-        | guidata.dataset.io.JSONReader,
+        reader: (
+            guidata.dataset.io.HDF5Reader
+            | guidata.dataset.io.INIReader
+            | guidata.dataset.io.JSONReader
+        ),
     ) -> None:
         """Deserialize object from HDF5 reader
 
@@ -1585,7 +1613,9 @@ class BasePlot(qwt.QwtPlot):
         item.unselect()
         self.SIG_ITEM_SELECTION_CHANGED.emit(self)
 
-    def get_last_active_item(self, item_type: itf.IItemType) -> itf.IBasePlotItem:
+    def get_last_active_item(
+        self, item_type: type[itf.IItemType]
+    ) -> itf.IBasePlotItem | None:
         """Return last active item corresponding to passed `item_type`
 
         Args:
